"""This Module defines the arguments that argparse will accept from the CLI"""
from json import loads

ARGUMENTS = {
    ############################################################################
    # Data structure containing all arguments a user could pass into pkpass
    # This helps us de-dupe our work and keep args consistent between
    # subcommands.  We use this with *args and **kwargs constructs
    ############################################################################
    'all': {
        'args': ['-a', '--all'],
        'kwargs': {
            'help': "Show all available password to the given user, if a pwname is supplied filtering will be done case-insensitivey based on the filename",
            'action': "store_true"
        }
    },

    'authorizer': {
        'args': ['--authorizer'],
        'kwargs': {
            'help': "The person or account authorizing the creation of this secret",
            'type': str
        }
    },

    'behalf': {
        'args': ['-b', '--behalf'],
        'kwargs': {
            'help': 'Show passwords for a user using a password as its private key',
            'type': str,
        }
    },

    'cabundle': {
        'args': ['--cabundle'],
        'kwargs': {
            'type': str,
            'help': "Path to CA certificate bundle file"
        }
    },

    'card_slot': {
        'args': ['-c', '--card_slot'],
        'kwargs': {
            'type': str,
            'help': 'The slot number of the card that should be used'
        }
    },

    'certpath': {
        'args': ['--certpath'],
        'kwargs': {
            'type': str,
            'help': "Path to directory containing public keys.  Certificates must end in '.cert'"
        }
    },

    'color': {
        'args': ['--color'],
        'kwargs': {
            'type': str,
            'help': "Disable color or not, accepts true/false"
        }
    },

    'connect': {
        'args': ['--connect'],
        'kwargs': {
            'type': loads,
            'help': "Connection string for the api to retrieve certs"
        }
    },

    'description': {
        'args': ['-d', '--description'],
        'kwargs': {
            'help': "A description of this secret",
            'type': str
        }
    },

    'escrow_users': {
        'args':['-e', '--escrow_users'],
        'kwargs': {
            'help': "Escrow users list is a comma sepearated list of recovery users that each get part of a key",
            'type': str
        },
    },

    'filter': {
        'args': ['-f', '--filter'],
        'kwargs': {
            'help': "Reduce output of commands to matching items",
            'type': str
        },
    },

    'groups': {
        'args': ['-g', '--groups'],
        'kwargs': {
            'type': str,
            'help': 'Comma seperated list of recipient groups'
        }
    },

    'identity': {
        'args': ['-i', '--identity'],
        'kwargs': {
            'type': str,
            'help': 'Override identity of user running the program'
        }
    },

    'ignore_decrypt': {
        'args': ['-I', '--ignore-decrypt'],
        'kwargs': {
            'action': 'store_true',
            'help': 'Ignore decryption errors during show all process'
        }
    },

    'keypath': {
        'args': ['--keypath'],
        'kwargs': {
            'type': str,
            'help': "Path to directory containing private keys.  Keys must end in '.key'"
        }
    },

    'long_escrow':{
        'args': ['-l', '--long-escrow'],
        'kwargs': {
            'action': 'store_true',
            'help': 'Long list passwords that have been escrowed, only shows escrowed passwords'
        }
    },

    'min_escrow': {
        'args': ['-m', '--min_escrow'],
        'kwargs':{
            'type': int,
            'help': "Minimum number of users required to unlock escrowed password"
        }
    },

    'nocache': {
        'args': ['--no-cache'],
        'kwargs': {
            'action': 'store_true',
            'help': 'if using a connector, pull the certs again'
        }
    },

    'noescrow': {
        'args': ['--noescrow'],
        'kwargs':{
            'action': 'store_true',
            'help': 'Do not use escrow functionality, ignore defaults in rc file'
        }
    },

    'nocrypto': {
        'args': ['--nocrypto'],
        'kwargs': {
            'action': 'store_true',
            'help': 'Do not use a password for import/export files'
        }
    },

    'nopassphrase': {
        'args': ['--nopassphrase', '--nopin'],
        'kwargs': {
            'action': 'store_true',
            'help': 'Do not prompt for a pin/passphrase'
        }
    },

    'nosign': {
        'args': ['--nosign'],
        'kwargs': {
            'action': 'store_true',
            'help': 'Do not digitally sign the password information that you are generating'
        }
    },

    'noverify': {
        'args': ['--noverify'],
        'kwargs': {
            'action': 'store_true',
            'help': 'Do not verify certificates and signatures'
        }
    },

    'overwrite': {
        'args': ['--overwrite'],
        'kwargs': {
            'action': 'store_true',
            'help': 'Overwrite a password that already exists'
        }
    },

    'pwfile': {
        'args': ['pwfile'],
        'kwargs': {
            'type': str,
            'help': 'path to the import/export file',
            'nargs': '?',
            'default': None
        }
    },

    'pwname': {
        'args': ['pwname'],
        'kwargs': {
            'type': str,
            'help': 'Name of the password. Ex: passwords/team/infrastructure/root',
            'nargs': '?',
            'default': None
        }
    },

    'pwstore': {
        'args': ['--pwstore', '--srcpwstore'],
        'kwargs': {
            'type': str,
            'help': 'Path to the source password store.  Defaults to "./passwords"'
        }
    },

    'quiet': {
        'args': ['-q', '--quiet'],
        'kwargs': {
            'action': 'store_const',
            'const': -1,
            'default': 0,
            'dest': 'verbosity',
            'help': 'quiet output (show errors only)'
        }
    },

    'recovery': {
        'args': ['-r', '--recovery'],
        'kwargs': {
            'action': 'store_true',
            'help': 'Work with passwords distributed through escrow functionality'
        }
    },

    'rename': {
        'args': ['rename'],
        'kwargs': {
            'type': str,
            'help': 'New name of the password.',
            'nargs': '?',
            'default': None
        }
    },

    'rules' :{
        'args': ['-R', '--rules'],
        'kwargs': {
            'type': str,
            'help': 'Key of rules to use from provided rules map'
        }
    },

    'rules_map' :{
        'args': ['--rules-map'],
        'kwargs': {
            'type': loads,
            'help': 'Map of rules used for automated generation of passwords'
        }
    },

    'stdin': {
        'args': ['--stdin'],
        'kwargs': {
            'action': 'store_true',
            'help': 'Take all password input from stdin instead of from a user input prompt'
        }
    },

    'time': {
        'args': ['-t', '--time'],
        'kwargs': {
            'type': int,
            'help': 'Number of seconds to keep password in paste buffer'
        }
    },

    'theme_map': {
        'args': ['--theme-map'],
        'kwargs': {
            'type': loads,
            'help': 'Map of colors to use for colorized output'
        }
    },

    'users': {
        'args': ['-u', '--users'],
        'kwargs': {
            'type': str,
            'help': 'Comma seperated list of recipients'
        }
    },

    'verbosity': {
        'args': ['-v', '--verbose'],
        'kwargs': {
            'action': 'count',
            'dest': 'verbosity',
            'default': 0,
            'help': 'verbose output (repeat for increased verbosity)'
        }
    }
}



#!/usr/bin/env python3
"""This module tests the modify module"""
import builtins
import unittest
import mock
import libpkpass.commands.cli as cli
import libpkpass.commands.modify as modify
from libpkpass.errors import DecryptionError
from .basetest.basetest import patch_args

class ModifyTests(unittest.TestCase):
    """This class tests the modify class"""
    def test_modify_success(self):
        """Test modifying the metadata of a password"""
        ret = True
        try:
            with patch_args(subparser_name='modify', identity='r3', nopassphrase='true',
                            pwname='gentest'):
                with mock.patch.object(builtins, 'input', lambda _: 'y'):
                    modify.Modify(cli.Cli())
        except DecryptionError:
            ret = False
        self.assertTrue(ret)



#!/usr/bin/env python3
"""General Utility file for common functionality"""
from os import path
from json import loads
from getpass import getuser
from sys import argv
from re import search, error
from fnmatch import filter as fnfilter
from argparse import _SubParsersAction
from yaml import safe_load
from yaml.parser import ParserError
from yaml.scanner import ScannerError
from colored import fg, attr
from libpkpass import __version__
from libpkpass.errors import FileOpenError, JsonArgumentError, ConfigParseError

    ####################################################################
def color_prepare(string, color_type, colorize, theme_map=None):
    """Handle the color output of a given string"""
    ####################################################################
    if theme_map is None:
        theme_map = {}
    color_defaults = {
        "info": "cyan",
        "warning": "yellow",
        "debug": "red",
        "first_level": "magenta",
        "second_level": "green",
        "third_level": "blue"
    }
    color = theme_map[color_type].lower() if (color_type in theme_map) else color_defaults[color_type]
    try:
        return "%s%s%s" % (fg(color), string, attr('reset')) if colorize else string
    except KeyError:
        return "%s%s%s" % (fg(color_defaults[color_type]), string, attr('reset')) if colorize else string


    ####################################################################
def set_default_subparser(self, name, args=None, positional_args=0):
    """Set default subparser to interpreter"""
    ####################################################################
    subparser_found = False
    for arg in argv[1:]:
        if arg in ['-h', '--help']:
            break
    else:
        # pylint: disable=protected-access
        # disabling protected access because subparsers are protected
        # and we want to be able to have a default subparser
        for action in self._subparsers._actions:
            if not isinstance(action, _SubParsersAction):
                continue
            for _ in argv[1:]:
                subparser_found = True
        if not subparser_found:
            if args is None:
                argv.insert(len(argv) - positional_args, name)
            else:
                args.insert(len(args) - positional_args, name)

    ####################################################################
def show_version():
    """return the version number in the VERSION file"""
    ####################################################################
    return __version__

    ####################################################################
def sort(lst):
    """Sort our alphanumeric keys"""
    ####################################################################
    lst = [str(i) for i in lst]
    lst.sort()
    return [int(i) if i.isdigit() else i for i in lst]

    ####################################################################
def dictionary_filter(string_match, dictionary, secondary_check=None):
    """Filter out our dictionary"""
    ####################################################################
    key_list = []
    key_list = fnfilter(dictionary.keys(), string_match)

    # If not a fileglob match, try regex
    if not key_list:
        try:
            key_list = [k for k in dictionary.keys() if search(string_match, k)]
        except error as err:
            print(err)

    if not secondary_check:
        return {k: dictionary[k] for k in key_list}
    return {k: dictionary[k] for k in key_list
            if secondary_check[0] in dictionary[k][secondary_check[1]].keys()}

    ##################################################################
def handle_filepath_args(args):
    """Does filepath expansion for config args"""
    ##################################################################
    file_path_args = ['cabundle', 'pwstore', 'certpath', 'keypath']
    for arg in file_path_args:
        if arg in args and args[arg]:
            args[arg] = path.expanduser(args[arg])
    if 'connect' in args and args['connect'] and \
            'base_directory' in args['connect'] and args['connect']['base_directory']:
        args['connect']['base_directory'] = path.expanduser(args['connect']['base_directory'])
    return args

    ##################################################################
def handle_boolean_args(args, argname):
    ##################################################################
    if isinstance(args[argname], str):
        return args[argname].upper() == 'TRUE'
    return args[argname]

    ##################################################################
def convert_strings_to_list(args, argname):
    """ convert argparsed strings to lists for an argument """
    ##################################################################
    if argname in args:
        args[argname] = args[argname].split(",") if args[argname] else []
        return [arg.strip() for arg in args[argname] if arg.strip()]
    return []

    ##################################################################
def parse_json_arguments(args, argument):
    """ Parses the json.loads arguments as dictionaries to use"""
    ##################################################################
    try:
        if argument in args and args[argument]:
            if isinstance(args[argument], dict):
                return args[argument]
            return loads(args[argument])
        return None
    except ValueError as err:
        raise JsonArgumentError(argument, err)

    ##################################################################
def collect_args(parsedargs):
    ##################################################################
    # Build a dict out of the argparse args Namespace object and a dict from any
    # configuration files and merge the two with cli taking priority
    args = {
        'ignore_decrypt': False,
        'identity': getuser(),
        'cabundle': './certs/ca-bundle',
        'keypath': './private',
        'pwstore': './passwords',
        'time': 10,
        'card_slot': None,
        'certpath': None,
        'escrow_users': None,
        'min_escrow': None,
        'no_cache': False,
        'noverify': None,
        'noescrow': False,
        'overwrite': False,
        'recovery': False,
        'rules': 'default',
        'stdin': False,
        'theme_map': None,
        'color': True,
        'verbosity': 0,
    }
    cli_args = parsedargs if isinstance(parsedargs, dict) else vars(parsedargs)
    config_args = get_config_args(cli_args['config'], cli_args)
    args.update(config_args)
    args['connect'] = parse_json_arguments(args, 'connect')
    args = handle_filepath_args(args)

    fles = ['cabundle', 'pwstore']
    for key, value in cli_args.items():
        if value is not None or key not in args:
            args[key] = value
        if key in fles and not path.exists(args[key]):
            raise FileOpenError(args[key], "No such file or directory")

    # json args
    args['color'] = handle_boolean_args(args, 'color')
    args['groups'] = convert_strings_to_list(args, 'groups')
    args['users'] = convert_strings_to_list(args, 'users')
    args['escrow_users'] = convert_strings_to_list(args, 'escrow_users')
    return args

    ##################################################################
def get_config_args(config, cli_args):
    """Return the configuration from the config file"""
    ##################################################################
    try:
        with open(config, 'r') as fname:
            config_args = safe_load(fname)
        return config_args if config_args else {}
    except IOError:
        if cli_args['verbosity'] != -1:
            print("INFO: No .pkpassrc file found")
        return {}
    except (ParserError, ScannerError):
        raise ConfigParseError("Parsing error with config file, please check syntax")



#!/usr/bin/env python3
"""This module tests the rename module"""
import builtins
import unittest
import mock
import libpkpass.commands.cli as cli
import libpkpass.commands.rename as rename
from libpkpass.errors import DecryptionError, CliArgumentError
from .basetest.basetest import patch_args, ERROR_MSGS

class RenameTests(unittest.TestCase):
    """This class tests the rename class"""
    def test_recipient_not_in_database(self):
        """Test what happens when a recipient is not in the appropriate directory"""
        with self.assertRaises(CliArgumentError) as context:
            with patch_args(subparser_name='rename', identity='bleh', nopassphrase='true',
                            all=None, pwname='test', rename='retest'):
                rename.Rename(cli.Cli())
        self.assertEqual(
            context.exception.msg,
            ERROR_MSGS['rep']
        )

    def test_rename_success(self):
        """Test a successful rename (then rename it back)"""
        ret = True
        try:
            with patch_args(subparser_name='rename', identity='r1', nopassphrase='true',
                            all=True, pwname='test', rename='retest', overwrite='true'):
                with mock.patch.object(builtins, 'input', lambda _: 'y'):
                    rename.Rename(cli.Cli())
            with patch_args(subparser_name='rename', identity='r1', nopassphrase='true',
                            all=True, pwname='retest', rename='test', overwrite="true"):
                with mock.patch.object(builtins, 'input', lambda _: 'y'):
                    rename.Rename(cli.Cli())
        except DecryptionError:
            ret = False
        self.assertTrue(ret)

    def test_rename_cli_error(self):
        """Test what happens when pwname is not supplied"""
        with self.assertRaises(CliArgumentError) as context:
            with patch_args(subparser_name='rename', identity='r1', nopassphrase='true',
                            all=True, rename='test', overwrite='true'):
                rename.Rename(cli.Cli())
        self.assertEqual(context.exception.msg, ERROR_MSGS['pwname'])

if __name__ == '__main__':
    unittest.main()



"""This module allows for the automated generation of passwords"""
from re import search
from re import error as re_error
from exrex import getone
from libpkpass.commands.command import Command
from libpkpass.errors import CliArgumentError, NotThePasswordOwnerError, RulesMapError
from libpkpass.util import parse_json_arguments

    ####################################################################
class Generate(Command):
    """This class implements the CLI functionality of automatic generation of passwords"""
    ####################################################################
    name = 'generate'
    description = 'Generate a new password entry and encrypt it for yourself'
    selected_args = Command.selected_args + ['pwname', 'pwstore', 'overwrite', 'stdin', 'keypath',
                                             'nopassphrase', 'nosign', 'card_slot', 'escrow_users',
                                             'min_escrow', 'noescrow', 'rules', 'rules_map', 'description',
                                             'authorizer']

        ####################################################################
    def _run_command_execution(self):
        """ Run function for class.                                      """
        ####################################################################
        safe, owner = self.safety_check()

        if safe or self.args['overwrite']:
            password = self._generate_pass()

            if 'description' not in self.args or not self.args['description']:
                self.args['description'] = input("Description: ")

            if 'authorizer' not in self.args or not self.args['authorizer']:
                self.args['authorizer'] = input("Authorizer: ")

            self.create_or_update_pass(password, self.args['description'], self.args['authorizer'])
        else:
            raise NotThePasswordOwnerError(self.args['identity'], owner, self.args['pwname'])

        ####################################################################
    def _generate_pass(self):
        """ Generate a password based on a rules list that has been provided"""
        #######################################################################
        try:
            regex_rule = self.args['rules_map'][self.args['rules']]
            password = getone(regex_rule)
            # The following regex search verifies it finds a match from exrex
            return search(regex_rule, password).group(0)
        except TypeError:
            raise RulesMapError("Poorly formatted Rules Map, please check it is in json format")
        except re_error:
            raise RulesMapError("Poorly formatted regex, or unsupported")

        #######################################################################
    def _validate_args(self):
        #######################################################################
        self.args['rules_map'] = parse_json_arguments(self.args, 'rules_map')
        for argument in ['pwname', 'keypath', 'rules_map']:
            if argument not in self.args or self.args[argument] is None:
                raise CliArgumentError(
                    "'%s' is a required argument" % argument)
        if self.args['rules'] not in self.args['rules_map'] or self.args['rules_map'][self.args['rules']] == "":
            raise RulesMapError("No Rule set defined as %s" % self.args['rules'])



#!/usr/bin/env python3
"""This module tests the listrecipients module"""
import unittest
import yaml
import libpkpass.commands.cli as cli
import libpkpass.commands.listrecipients as listrecipients
from .basetest.basetest import captured_output, patch_args

class ListrecipientsTests(unittest.TestCase):
    """This class tests the listrecipients class"""
    def setUp(self):
        self.out_dict = {
            'Certificate store': 'test/pki/intermediate/certs',
            'Key store': 'test/pki/intermediate/private',
            'CA Bundle file': 'test/pki/intermediate/certs/ca-bundle',
            'Looking for Key Extension': '.key',
            'Looking for Certificate Extension': "['.cert', '.crt']",
            'Loaded 4 identities': None,
            'r3': {
                'certs': {
                    'verified': True,
                }
            }
        }

    def test_listrecipients(self):
        """Test a listing of our recipients"""
        self.maxDiff = None
        with patch_args(subparser_name='listrecipients', identity='r1',
                        nopassphrase="true", filter="r3"):
            with captured_output() as (out, _):
                listrecipients.Listrecipients(cli.Cli())
        output = yaml.safe_load(out.getvalue().replace('\t', '  '))
        # we remove these things because the actual values depend on creation
        # moreover, some of the outputs on different operating systems appear
        # to utilize different delimiters.
        del output['r3']['certs']['enddate']
        del output['r3']['certs']['subjecthash']
        del output['r3']['certs']['issuerhash']
        del output['r3']['certs']['fingerprint']
        del output['r3']['certs']['subject']
        del output['r3']['certs']['issuer']
        self.assertDictEqual(output, self.out_dict)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
"""This module tests the list module"""
import unittest
import yaml
import libpkpass.commands.cli as cli
import libpkpass.commands.list as pklist
from libpkpass.errors import CliArgumentError
from .basetest.basetest import captured_output, patch_args, ERROR_MSGS

class ListTests(unittest.TestCase):
    """This class tests the list class"""
    def setUp(self):
        self.password_list_0 = {
            "Passwords for 'r2'": None
        }
        self.password_list_1 = {
            "Passwords for 'r1'": None,
            'test/passwords/test': {
                'Distributor': 'r1',
                'Name': 'test'
            }
        }

    def test_recipient_not_in_database(self):
        """test bad recipient functionality"""
        with self.assertRaises(CliArgumentError) as context:
            with patch_args(subparser_name='list', identity='bleh', nopassphrase="true"):
                pklist.List(cli.Cli())
        self.assertEqual(
            context.exception.msg,
            ERROR_MSGS['rep']
        )

    def test_list_none(self):
        """test list functionality for no passwords"""
        with patch_args(subparser_name='list', identity='r2', nopassphrase="true"):
            with captured_output() as (out, _):
                pklist.List(cli.Cli())
        output = yaml.safe_load(out.getvalue().replace('\t', ' '))
        self.assertDictEqual(output, self.password_list_0)

    def test_list_one(self):
        """test list functionality for one password"""
        with patch_args(subparser_name='list', identity='r1', nopassphrase="true"):
            with captured_output() as (out, _):
                pklist.List(cli.Cli())
        output = yaml.safe_load(out.getvalue().replace('\t', ' '))
        self.assertDictEqual(output, self.password_list_1)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
"""This module tests the create module"""
import getpass
import builtins
import unittest
import mock
import libpkpass.commands.cli as cli
import libpkpass.commands.create as create
from libpkpass.errors import CliArgumentError, DecryptionError
from .basetest.basetest import patch_args, ERROR_MSGS

class CreateTests(unittest.TestCase):
    """This class tests the create class"""
    def test_create_success(self):
        """Test a success creation of a password"""
        ret = True
        try:
            with patch_args(subparser_name='create', identity='r1', nopassphrase='true',
                            pwname='test'):
                with mock.patch.object(builtins, 'input', lambda _: 'y'):
                    with mock.patch.object(getpass, 'getpass', lambda _: 'y'):
                        create.Create(cli.Cli())
        except DecryptionError:
            ret = False
        self.assertTrue(ret)

    def test_create_no_pass(self):
        """Test what happens with pwname is not supplied"""
        with self.assertRaises(CliArgumentError) as context:
            with patch_args(subparser_name='create', identity='r1', nopassphrase='true',
                            pwname=None):
                with mock.patch.object(builtins, 'input', lambda _: 'y'):
                    with mock.patch.object(getpass, 'getpass', lambda _: 'y'):
                        create.Create(cli.Cli())
        self.assertEqual(context.exception.msg, ERROR_MSGS['pwname'])

if __name__ == '__main__':
    unittest.main()



"""This module allows for opening a PkPass interactive interpreter"""
# Look, I know what I did; I'm sorry about the following file.
# I pray that there exists a better way to do this. This file
# Should be burned to the ground and never spoken of again.
# You know it, I know it, everybody knows it.

# We need to disable unused import linting because the command classes *are* actually used...
# pylint: disable=unused-import
from os import path, environ, chdir, sep
import sys
from glob import glob
from subprocess import call
from cmd import Cmd
from inspect import getmembers, isclass
from yaml.parser import ParserError
from libpkpass.commands.command import Command
from libpkpass.errors import PKPassError, JsonArgumentError
from libpkpass.identities import IdentityDB
from libpkpass.util import handle_filepath_args, show_version, collect_args
import libpkpass.commands.card as card
import libpkpass.commands.clip as clip
import libpkpass.commands.create as create
import libpkpass.commands.delete as delete
import libpkpass.commands.distribute as distribute
import libpkpass.commands.export as export
import libpkpass.commands.fileimport as pkimport
import libpkpass.commands.generate as generate
import libpkpass.commands.info as info
import libpkpass.commands.list as pklist
import libpkpass.commands.listrecipients as listrecipients
import libpkpass.commands.modify as modify
import libpkpass.commands.pkinterface as pkinterface
import libpkpass.commands.recover as recover
import libpkpass.commands.rename as rename
import libpkpass.commands.show as show
import libpkpass.commands.update as update

VERSION = show_version()

    ####################################################################
class Interpreter(Command):
    """This class implements a skeleton to call the interpreter"""
    name = 'interpreter'
    description = 'Interactive mode for pkpass'
    selected_args = Command.selected_args + ['card_slot', 'connect', 'escrow_users', 'min_escrow',
                                             'groups', 'keypath', 'pwstore']
    ####################################################################

        ####################################################################
    def _run_command_execution(self):
        """ Run function for class. """
        ####################################################################
        Interactive(self.args, self.identities).cmdloop_with_keyboard_interrupt()

        ####################################################################
    def _validate_args(self):
        ####################################################################
        pass

# This is to keep argparse from sys.exiting all over the place
    ####################################################################
def pkparse_error(message):
    ####################################################################
    raise PKPassError(message)

    ####################################################################
class Interactive(Cmd, pkinterface.PkInterface):
    """This class implements the interactive interpreter functionality"""
    intro = """Welcome to PKPass (Public Key Based Password Manager) v%s!
Type ? to list commands""" % VERSION
    prompt = 'pkpass> '
    ####################################################################

        ####################################################################
    def __init__(self, args, recipients_database):
        ####################################################################
        Cmd.__init__(self)
        # manually remove interpreter from command line so that argparse doesn't try to use it
        # This needs to be removed before the PkInterace init
        sys.argv.remove('interpreter')

        pkinterface.PkInterface.__init__(self)

        self.recipients_database = recipients_database if recipients_database else IdentityDB()
        self.parser.error = pkparse_error

        # Hold onto args passed on the command line
        self.args = args
        # change our cwd so users can tab complete
        self._change_pwstore()

        # We basically need to grab the first argument and ditch it
        self.parsedargs = {}

        ####################################################################
    def cmdloop_with_keyboard_interrupt(self):
        """handle keyboard interrupts"""
        ####################################################################
        breaker = False
        first = True
        while breaker is not True:
            try:
                if first:
                    self.cmdloop()
                else:
                    self.cmdloop(intro='')
                breaker = True
            except KeyboardInterrupt:
                first = False
                sys.stdout.write('\n')

        ####################################################################
    def _load_iddb(self):
        """load recipient database"""
        ####################################################################
        self.recipients_database = IdentityDB()
        self.recipients_database.cabundle = self.args['cabundle']
        self.recipients_database.load_certs_from_directory(
            self.args['certpath'],
            verify_on_load=True,
            connectmap=self.args['connect'],
            nocache=False
        )
        if 'keypath' in self.args:
            self.recipients_database.load_keys_from_directory(self.args['keypath'])

        ####################################################################
    def _change_pwstore(self):
        """Change directory"""
        ####################################################################
        direc = self.args['pwstore']
        if 'pwstore' in self.args and direc and path.isdir(direc):
            chdir(direc)

        ####################################################################
    def _reload_config(self):
        """Change affected globals in interpreter"""
        ####################################################################
        # We still need to be able to reload other things like recipients
        # database
        config = self.args['config']
        config_args = self.args
        try:
            config_args = collect_args({'config': config})
            if not config_args:
                config_args = self.args
        except ParserError:
            print("Error parsing config file")
            config_args = self.args
        finally:
            self.args = config_args
            self.args['config'] = config
            self.args = handle_filepath_args(self.args)
            self._change_pwstore()
            self._load_iddb()

        ####################################################################
    def precmd(self, line):
        """ Fix command line arguments, this is a hack to allow argparse
        function as we expect """
        ####################################################################
        sys.argv = [sys.argv[0]]
        sys.argv.extend(line.split())
        return line

        ####################################################################
    def postcmd(self, stop, line):
        """ Fix command line arguments, this is a hack to allow argparse
        function as we expect """
        ####################################################################
        if str(line) == "edit" or '--no-cache' in line:
            self._reload_config()
        return Cmd.postcmd(self, stop, line)

        ####################################################################
    def _append_slash_if_dir(self, path_arg):
        """ Appending slashes for autocomplete_file_path """
        ####################################################################
        if path_arg and path.isdir(path_arg) and path_arg[-1] != sep:
            return path_arg + sep
        return path_arg

        ####################################################################
    def autocomplete_file_path(self, _, line, begidx, endidx):
        """ File path autocompletion, used with the cmd module complete_* series functions"""
        ####################################################################
        before_arg = line.rfind(" ", 0, begidx)
        if before_arg == -1:
            return None # arg not found

        fixed = line[before_arg+1:begidx]  # fixed portion of the arg
        arg = line[before_arg+1:endidx]
        pattern = arg + '*'

        completions = []
        for fpath in glob(pattern):
            fpath = self._append_slash_if_dir(fpath)
            completions.append(fpath.replace(fixed, "", 1))
        return completions

        ####################################################################
    def default(self, line):
        """If we don't have a proper subcommand passed"""
        ####################################################################
        print("Command '%s' not found, see help (?) for available commands"
              % line)
        return False

        ####################################################################
    def do_exit(self, _):
        """Exit the application"""
        ####################################################################
        return True

        ####################################################################
    def do_version(self, _):
        """Print the version of PkPass"""
        ####################################################################
        print(VERSION)

        ####################################################################
    def do_edit(self, _):
        """Edit your configuration file, with $EDITOR"""
        ####################################################################
        try:
            call([environ['EDITOR'], self.args['config']])
            return False
        except (IOError, SystemExit):
            return False

        ####################################################################
    def do_git(self, line):
        """If your password store is git back, you can control git from here"""
        ####################################################################
        call(["git"] + line.split())
        return False

    ##############################################################################
def add_dynamic_function(module_name, class_name):
    """This allows repetitive do_* and complete_* functions to be condensed"""
    ##############################################################################
    swap_name = module_name
    if "pk" in module_name:
        swap_name = module_name[2:]
    fn_name = "do_" + swap_name
    complete_name = "complete_" + swap_name

        ############################################
    def do_fn(self, _):
        """Use -h or --help for information"""
        ############################################
        try:
            pk_class = [o for o in getmembers(globals()[module_name], isclass)
                        if str(o[0]) == class_name][0][1]
            pk_class(self, iddb=self.recipients_database)
            self.actions[swap_name].run(self.parser.parse_args())
            return False
        except PKPassError as err:
            print(err)
            return False
        except SystemExit:
            return False

        ##################################################
    def complete_fn(self, text, line, begidx, endidx):
        ##################################################
        return self.autocomplete_file_path(text, line, begidx, endidx)

    setattr(Interactive, fn_name, do_fn)
    setattr(Interactive, complete_name, complete_fn)

for command in [('card', 'Card'), ('clip', 'Clip'), ('create', 'Create'), ('delete', 'Delete'),
                ('distribute', 'Distribute'), ('export', 'Export'),
                ('generate', 'Generate'), ('pkimport', 'Import'), ('info', 'Info'),
                ('pklist', 'List'), ('listrecipients', 'Listrecipients'),
                ('modify', 'Modify'), ('recover', 'Recover'), ('rename', 'Rename'),
                ('show', 'Show'), ('update', 'Update')]:
    add_dynamic_function(command[0], command[1])



"""This module is used to process the decryption of keys"""
from os import path, unlink, walk
from fnmatch import fnmatch
from tempfile import gettempdir
from libpkpass.commands.command import Command
from libpkpass.password import PasswordEntry
from libpkpass.errors import PasswordIOError, CliArgumentError, NotARecipientError, DecryptionError

    ####################################################################
class Show(Command):
    """This class is used as a command object and parses information passed through
    the CLI to show passwords that have been distributed to users"""
    ####################################################################
    name = 'show'
    description = 'Display a password'
    selected_args = Command.selected_args + [
        'all', 'behalf', 'card_slot', 'ignore_decrypt', 'keypath', 'nopassphrase',
        'noverify', 'pwname', 'pwstore', 'recovery', 'stdin'
    ]
        ####################################################################
    def _run_command_execution(self):
        """ Run function for class.                                  """
        ####################################################################
        password = PasswordEntry()
        myidentity = self.identities.iddb[self.args['identity']]
        if 'behalf' in self.args and self.args['behalf']:
            self._behalf_prep(password, myidentity)
        else:
            self._show_wrapper(password, myidentity)

        ####################################################################
    def _show_wrapper(self, password, myidentity):
        """Wrapper for show to allow for on behalf of"""
        ####################################################################
        if self.args['all']:
            try:
                self._walk_dir(self.args['pwstore'], password, myidentity, self.args['ignore_decrypt'])
            except DecryptionError as err:
                raise err
        elif self.args['pwname'] is None:
            raise PasswordIOError("No password supplied")
        else:
            self._decrypt_wrapper(
                self.args['pwstore'], password, myidentity, self.args['pwname'])

        ####################################################################
    def _behalf_prep(self, password, myidentity):
        """Create necessary temporary file for rsa key"""
        ####################################################################
        password.read_password_data(path.join(self.args['pwstore'], self.args['behalf']))
        # allows the password to be stored outside the root of the password directory
        self.args['behalf'] = self.args['behalf'].split('/')[-1]
        temp_key = path.join(gettempdir(), '%s.key' % self.args['behalf'])
        plaintext_pw = password.decrypt_entry(
            identity=myidentity, passphrase=self.passphrase, card_slot=self.args['card_slot'])
        with open(temp_key, 'w') as fname:
            fname.write(
                '%s\n%s\n%s' % (
                    '-----BEGIN RSA PRIVATE KEY-----',
                    plaintext_pw.replace(
                        '-----BEGIN RSA PRIVATE KEY-----', ''
                    ).replace(
                        ' -----END RSA PRIVATE KEY----', ''
                    ).replace(' ', '\n').strip(),
                    '-----END RSA PRIVATE KEY-----'
                )
            )
        self.args['identity'] = self.args['behalf']
        myidentity = self.identities.iddb[self.args['identity']]
        self.args['key_path'] = temp_key
        myidentity['key_path'] = temp_key
        print(myidentity)
        self._show_wrapper(password, myidentity)
        unlink(temp_key)

        ####################################################################
    def _walk_dir(self, directory, password, myidentity, ignore_decrypt=False):
        """Walk our directory searching for passwords"""
        ####################################################################
        # walk returns root, dirs, and files we just need files
        for root, _, pwnames in walk(directory):
            for pwname in pwnames:
                if self.args['pwname'] is None or\
                        fnmatch(path.join(root, pwname), self.args['pwname']):
                    try:
                        self._decrypt_wrapper(
                            root, password, myidentity, pwname)
                    except DecryptionError as err:
                        if ignore_decrypt:
                            print(err)
                            continue
                        raise
                    except NotARecipientError:
                        continue

        ####################################################################
    def _handle_escrow_show(self, password, myidentity):
        """This populates the user's escrow as passwords"""
        ####################################################################
        myescrow = []
        if password.escrow:
            for key, value in password['escrow'].items():
                if myidentity['uid'] in value['recipients'].keys():
                    myescrow.append([value['recipients'][myidentity['uid']], key])
        return myescrow

        ####################################################################
    def _decrypt_wrapper(self, directory, password, myidentity, pwname):
        """Decide whether to decrypt normally or for escrow"""
        ####################################################################
        password.read_password_data(path.join(directory, pwname))
        myescrow = []
        if self.args['recovery']:
            myescrow = self._handle_escrow_show(password, myidentity)
        if myescrow:
            for share in myescrow:
                password['recipients'][myidentity['uid']] = share[0]
                print("Share for escrow group: %s" % share[1])
                self._decrypt_password_entry(password, myidentity)
        else:
            self._decrypt_password_entry(password, myidentity)

        ####################################################################
    def _decrypt_password_entry(self, password, myidentity):
        """This decrypts a given password entry"""
        ####################################################################
        plaintext_pw = password.decrypt_entry(
            identity=myidentity, passphrase=self.passphrase, card_slot=self.args['card_slot'])
        if not self.args['noverify']:
            result = password.verify_entry(
                myidentity['uid'], self.identities)
            if not result['sigOK']:
                print("warning: could not verify that '%s' correctly signed your password entry." %
                      result['distributor'])
            if not result['certOK']:
                print("Warning: could not verify the certificate authenticity of user '%s'." %
                      result['distributor'])

        print(("%s: %s") % (self.color_print(password.metadata['name'],
                                             "first_level"), plaintext_pw))

        ####################################################################
    def _validate_args(self):
        ####################################################################
        for argument in ['keypath']:
            if argument not in self.args or self.args[argument] is None:
                raise CliArgumentError(
                    "'%s' is a required argument" % argument)



#!/usr/bin/env python3
"""This Module handles the escrow functions i.e. creating shares and recovery"""
from pyseltongue import PlaintextToHexSecretSharer as ptohss
from .errors import EscrowError

    ##############################################################################
def pk_split_secret(plaintext_string, escrow_list, minimum=None):
    """split a secret into multiple shares for encryption"""
    ##############################################################################
    escrow_len = len(escrow_list)
    if not minimum and escrow_len % 2 != 0:
        minimum = int((escrow_len + 1) / 2)
    elif not minimum:
        minimum = int((escrow_len / 2) + 1)
    if minimum < 2:
        raise EscrowError("minimum escrow", 2, minimum)
    if escrow_len < 3:
        raise EscrowError("escrow users list", 3, escrow_len)
    return ptohss.split_secret(str(plaintext_string), minimum, escrow_len)

    ##############################################################################
def pk_recover_secret(shares):
    """Take Decrypted strings from crypto and recover a secret"""
    ##############################################################################
    return ptohss.recover_secret(shares)



#!/usr/bin/env python3
"""This Module tests the password entry module"""
import unittest
from libpkpass.password import PasswordEntry
from libpkpass.identities import IdentityDB

class TestBasicFunction(unittest.TestCase):
    """This Class tests the password entry class"""
    def setUp(self):
        self.certdir = 'test/pki/intermediate/certs'
        self.keydir = 'test/pki/intermediate/private'
        self.cabundle = 'test/pki/intermediate/certs/ca-bundle'
        self.file1 = 'test/passwords/testpassword'
        self.file2 = 'test/scratch/testpassword'
        self.secret = 'Secret'
        self.textblob = 'Testing TextField'
        self.sender = 'r1'

        self.idobj = IdentityDB()
        self.idobj.identity = self.sender
        self.idobj.recipient_list = ['r2', 'r3']
        self.idobj.load_certs_from_directory(self.certdir, self.cabundle)
        self.idobj.load_keys_from_directory(self.keydir)

    def test_create_encrypt_decrypt(self):
        """create a password entry"""
        passwordentry = PasswordEntry(name='testcreate',
                                      description=self.textblob,
                                      creator='r1',
                                      authorizer='r1')

        passwordentry.add_recipients(secret=self.secret,
                                     distributor='r1',
                                     recipients=self.idobj.recipient_list,
                                     identitydb=self.idobj)

    def test_read_write(self):
        """Read and write password entry data"""
        passwordentry = PasswordEntry()
        passwordentry.read_password_data(self.file1)
        passwordentry.write_password_data(self.file2, overwrite=True)

        with open(self.file1, 'r') as file1:
            with open(self.file2, 'r') as file2:
                self.assertTrue(file1.read() == file2.read())

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# Note: to use the 'upload' functionality of this file, you must:
# $ pip install twine
import os
import shutil
from shutil import rmtree
import sys
from subprocess import Popen, PIPE, STDOUT
import getpass
from setuptools import setup, Command
import versioneer

REQUIRED = [
    'PyYAML>=4.2b1',
    'colored>=1.4.0',
    'cryptography>=2.3',
    'exrex>=0.10.5',
    'pem>=20.1.0',
    'pyperclip>=1.6.0',
    'pyseltongue>=0.3.1',
    'python-dateutil>=2.8.1',
    'setuptools>=41.2.0',
]

HOME = os.path.expanduser("~")
HERE = os.path.abspath(os.path.dirname(__file__))

def rcfile_check():
    rc_location = os.path.join(HOME, ".pkpassrc")
    example_location = os.path.join(HERE, 'example_pkpassrc')
    if os.path.exists(rc_location):
        if os.path.isdir(rc_location):
            print("WARN: default rc file location is a directory")
        else:
            print(".pkpassrc already defined, skipping")
            return
    else:
        print("Copying example rc file to %s" % rc_location)
        shutil.copyfile(example_location, rc_location)

# Load the package's __version__.py module as a dictionary.
ABOUT = {}

class UploadCommand(Command):
    """Support setup.py upload."""
    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(string):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(string))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds..')
            rmtree(os.path.join(HERE, 'dist'))
        except OSError:
            pass

        self.status('Building source and wheel distribution..')
        os.system('{0} setup.py sdist bdist_while --universal'.format(sys.executable))

        self.status('Uploading the package to pypi via twine..')
        os.system('twine upload dist/*')

        self.status('Pushing git tags..')
        os.system('git tag v{0}'.format(ABOUT['__version__']))
        os.system('git push --tags')

        sys.exit()

class RCFile(Command):
    """Re/Create RCFile"""
    description = 'Create a pkpass rc file'
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def user_input(self, prompt, default):
        u_input = input(prompt).strip()
        return u_input if u_input else default

    def directory_creation(self, user_prompt, default):
        directory = self.user_input(user_prompt, default) if user_prompt else default
        directory = os.path.expanduser(directory)
        if not os.path.isdir(directory):
            os.makedirs(directory)
        return str(directory)

    def file_creation(self, user_prompt, default):
        filename = self.user_input(user_prompt, default)
        filename = os.path.expanduser(filename)
        if not os.path.exists(filename):
            self.directory_creation(None, os.path.dirname(filename))
            with open(filename, 'w+') as newfile:
                newfile.write("")
        return str(filename)

    def finish_run(self):
        testing = """testing versions of openssl and pkcs15-tool if version numbers return you're probably good
for sanity sake Noah's return values were:
    openssl version: LibreSSL 2.2.7
    pkcs15-tool --version: OpenSC-0.18.0, rev: eb60481f, commit-time: 2018-05-16 13:48:37 +0200
------YOUR VALUES BELOW THIS LINE -----------"""

        print(testing)
        print(Popen("openssl version".split(), stdout=PIPE, stdin=PIPE, stderr=STDOUT).communicate()[0].decode("ASCII"))
        print(Popen("pkcs15-tool --version".split(), stdout=PIPE, stdin=PIPE, stderr=STDOUT).communicate()[0].decode("ASCII"))

    def run(self):
        args = {
            'cabundle': '',
            'card_slot': '',
            'certpath': '',
            'connect': '',
            'escrow_users': '',
            'groups': '',
            'identity': '',
            'keypath': '',
            'min_escrow': '',
            'pwstore': '',
            'time': '',
            'rules': '',
            'rules_map': '',
            'users': ''
            }
        print("If not using defaults for the following paths please use full filepath or relative to home using ~")
        passdb_home = os.path.join(HOME, "passdb")
        args['certpath'] = self.directory_creation("Directory for certpath (defaults to ~/passdb/certs): ",
                                                   os.path.join(passdb_home, "certs"))
        args['keypath'] = self.directory_creation("Directory for keypath (defaults to ~/passdb/keys): ",
                                                  os.path.join(passdb_home, "keys"))
        args['cabundle'] = self.file_creation("Path to cabundle (defaults to ~/passdb/cabundles/ca.bundle): ",
                                              os.path.join(passdb_home, "cabundles", "ca.bundle"))
        args['pwstore'] = self.directory_creation("Directory for passwords (defaults to ~/passdb/passwords): ",
                                                  os.path.join(passdb_home, "passwords"))

        print(Popen("pkcs11-tool -L".split(), stdout=PIPE, stdin=PIPE, stderr=STDOUT).communicate()[0].decode("ASCII"))
        args['card_slot'] = self.user_input("Available slots listed above, which would you like to use? (defaults to 0): ", "0")
        args['identity'] = self.user_input("What user name would you like to use? (defaults to system user): ", getpass.getuser())
        print("Escrow users is a feature of Pkpass. Escrow allows a password to be recovered by the majority of the escrow users in the event of an emergency.")
        check_escrow = self.user_input("Would you like to setup escrow? ", "n")
        if check_escrow.lower()[0] == 'y':
            escrow_users = self.user_input("Please enter a comma seperated list of usernames: ", "").split(',')
            args['escrow_users'] = ",".join([user.strip() for user in escrow_users])
            args['min_escrow'] = self.user_input("What should be the minimum number of escrow users required to unlock? ",
                                                 len(args['escrow_users'].split(',')) - 1)

        with open(os.path.join(HOME, '.pkpassrc'), 'w') as fname:
            # fname.write(contents)
            for key, value in args.items():
                if value:
                    fname.write("%s: %s\n" % (key, value))
                else:
                    fname.write("#%s:\n" % key)

        self.finish_run()

class Verify(Command):
    """Verify the RC file is still valid
    use this with:
        python setup.py verify -r "/path/to/.pkpassrc" """
    description = 'Lints the pkpassrc file'
    user_options = [("rcfile=", "r", 'Specify the rc file path')]

    def initialize_options(self):
        self.rcfile = "~/.pkpassrc"

    def finalize_options(self):
        pass

    def check_if_recipient(self, user, certpath):
        for fname in os.listdir(certpath):
            if fname.endswith(tuple(['.crt', '.cert'])):
                uid = fname.split('.')[0]
                if uid == user:
                    return True
        return False

    def check_users(self, args_dict, certpath, valid):
        args = ['escrow_users', 'groups', 'identity', 'users']
        for arg in args:
            users_list = []
            if arg in args_dict.keys():
                if args_dict[arg]:
                    users_list = args_dict[arg].strip().split(',')
                    for user in users_list:
                        if not self.check_if_recipient(user.strip(), certpath):
                            valid = False
                            print("'%s' found in config, not in recipientsdb" % user)
                else:
                    valid = False
                    print("'%s' found in config, but value is empty" % arg)
        return valid

    def check_paths(self, args_dict, valid):
        args = ['cabundle', 'certpath', 'pwstore']
        for arg in args:
            if arg in args_dict.keys():
                if not os.path.exists(args_dict[arg]):
                    valid = False
                    print("'%s' found in config, No such file or directory: %s" %
                          (arg, args_dict[arg]))
        return valid

    def run(self):
        import yaml
        valid = True
        args = ['cabundle', 'card_slot', 'certpath',
                'connect', 'escrow_users',
                'groups', 'identity', 'keypath',
                'min_escrow', 'pwstore', 'time',
                'rules', 'rules_map', 'users',
                'theme_map', 'color']
        store_args = ['all', 'ignore_decrypt', 'long_escrow',
                      'noescrow', 'nocrypto', 'nopassphrase',
                      'nosign', 'noverify', 'overwrite',
                      'pwfile', 'pwname', 'recovery', 'stdin']
        args_dict = {}
        if self.rcfile:
            with open(os.path.expanduser(self.rcfile), 'r') as rcyaml:
                try:
                    args_dict = yaml.safe_load(rcyaml)
                except yaml.YAMLError as err:
                    valid = False
                    print(err)
        for arg in store_args:
            if arg in args_dict.keys():
                valid = False
                print("'%s' found in rc file, this will be ignored" % arg)

        for arg in args_dict.keys():
            if arg not in args and arg not in store_args:
                valid = False
                print("'%s' found in rc file, does not appear to be valid argument\
(It may be a user defined group)" % arg)

        if 'certpath' in args_dict.keys():
            valid = self.check_users(args_dict, args_dict['certpath'], valid)

        valid = self.check_paths(args_dict, valid)
        if valid:
            print("Config Valid")

CMDCLASS = versioneer.get_cmdclass()
CMDCLASS['upload'] = UploadCommand
CMDCLASS['rcfile'] = RCFile
CMDCLASS['verify'] = Verify
setup(
    version=versioneer.get_version(),
    install_requires=REQUIRED,
    extras_require={
        'testing': ["mock", "tox"]
    },
    cmdclass=CMDCLASS,
    )

try:
    rcfile_check()
except FileNotFoundError as err:
    print("WARN: could not copy example rc file")



"""This module allows for the creation of passwords"""
import getpass
from sys import stdin
from libpkpass.commands.command import Command
from libpkpass.errors import CliArgumentError, PasswordMismatchError, BlankPasswordError

    ####################################################################
class Create(Command):
    """This class implements the CLI functionality of creation of passwords"""
    ####################################################################
    name = 'create'
    description = 'Create a new password entry and encrypt it for yourself'
    selected_args = Command.selected_args + ['pwname', 'pwstore', 'overwrite', 'stdin', 'keypath',
                                             'nopassphrase', 'nosign', 'card_slot', 'escrow_users',
                                             'min_escrow', 'noescrow', 'description', 'authorizer']

        ####################################################################
    def _run_command_execution(self):
        """ Run function for class.                                      """
        ####################################################################

        if not self.args['stdin']:
            password1 = getpass.getpass("Enter password to create: ")
            if password1.strip() == "":
                raise BlankPasswordError
            password2 = getpass.getpass("Enter password to create again: ")
            if password1 != password2:
                raise PasswordMismatchError
        else:
            password1 = stdin.read()

        if 'description' not in self.args or not self.args['description']:
            self.args['description'] = input("Description: ")

        if 'authorizer' not in self.args or not self.args['authorizer']:
            self.args['authorizer'] = input("Authorizer: ")

        self.create_or_update_pass(password1, self.args['description'], self.args['authorizer'])
        ####################################################################
    def _validate_args(self):
        ####################################################################
        for argument in ['pwname', 'keypath']:
            if argument not in self.args or self.args[argument] is None:
                raise CliArgumentError(
                    "'%s' is a required argument" % argument)



"""This module allows for the renaming of passwords"""
import os
import sys
from libpkpass.commands.command import Command
from libpkpass.password import PasswordEntry
from libpkpass.errors import CliArgumentError, NotThePasswordOwnerError

    ####################################################################
class Rename(Command):
    """This class implements the CLI functionality of renaming of passwords"""
    ####################################################################
    name = 'rename'
    description = 'Rename a password in the repository'
    selected_args = Command.selected_args + ['pwname', 'pwstore', 'overwrite', 'stdin', 'nopassphrase',
                                             'keypath', 'card_slot', 'rename']

        ####################################################################
    def _run_command_execution(self):
        """ Run function for class. """
        ####################################################################
        safe, owner = self.safety_check()
        if safe and owner:
            orig_pass = self.args['pwname']
            self.args['pwname'] = self.args['rename']
            resafe, reowner = self.safety_check()
            self.args['pwname'] = orig_pass
            if resafe or self.args['overwrite']:
                myidentity = self.identities.iddb[self.args['identity']]
                password = PasswordEntry()
                password.read_password_data(os.path.join(self.args['pwstore'], self.args['pwname']))
                plaintext_pw = password.decrypt_entry(
                    identity=myidentity, passphrase=self.passphrase, card_slot=self.args['card_slot'])
                self._confirmation(plaintext_pw)
            else:
                raise NotThePasswordOwnerError(self.args['identity'], reowner, self.args['rename'])
        else:
            raise NotThePasswordOwnerError(self.args['identity'], owner, self.args['pwname'])

        ####################################################################
    def _confirmation(self, plaintext_pw):
        """Run confirmation for rename"""
        ####################################################################
        yes = {'yes', 'y', 'ye', ''}
        deny = {'no', 'n'}
        confirmation = input("%s: %s\nRename this password?(Defaults yes):"
                             % (self.args['pwname'], plaintext_pw))
        if confirmation.lower() in yes:
            self.rename_pass()
        elif confirmation.lower() in deny:
            sys.exit()
        else:
            print("please respond with yes or no")
            self._confirmation(plaintext_pw)

        ####################################################################
    def _validate_args(self):
        """Validate necessary arguments"""
        ####################################################################
        for argument in ['pwname', 'keypath', 'rename']:
            if argument not in self.args or self.args[argument] is None:
                raise CliArgumentError(
                    "'%s' is a required argument" % argument)



#!/usr/bin/env python3
"""This module tests the show module"""
import unittest
import libpkpass.commands.cli as cli
import libpkpass.commands.show as show
from libpkpass.errors import DecryptionError, CliArgumentError
from .basetest.basetest import ERROR_MSGS, patch_args

class ShowErrors(unittest.TestCase):
    """This class tests the show class"""
    def test_decryption(self):
        """Test successful decryption"""
        ret = True
        try:
            with patch_args(subparser_name='show', identity='r1', nopassphrase='true',
                            all=None, pwname='test'):
                show.Show(cli.Cli())
        except DecryptionError:
            ret = False
        self.assertTrue(ret)

    def test_recipient_not_in_database(self):
        """test what happens when the recipient is not in the appropriate directory"""
        with self.assertRaises(CliArgumentError) as context:
            with patch_args(subparser_name='show', identity='bleh', nopassphrase='true',
                            all=None, pwname='test'):
                show.Show(cli.Cli())
        self.assertEqual(
            context.exception.msg,
            ERROR_MSGS['rep']
        )

    def test_showall_decryption(self):
        """Test showing all passwords"""
        ret = True
        try:
            with patch_args(subparser_name='show', identity='r1', nopassphrase='true',
                            all=True, pwname='*test*'):
                show.Show(cli.Cli())
        except DecryptionError:
            ret = False
        self.assertTrue(ret)

    def test_show_nopass_error(self):
        """Test what happens when neither pwname or the all flag are supplied"""
        ret = False
        try:
            with patch_args(subparser_name='show', identity='r1', nopassphrase='true',
                            all=None):
                show.Show(cli.Cli())
        except KeyError as error:
            if str(error) == "'pwname'":
                ret = str(error)
        self.assertEqual(ret, "'pwname'")

if __name__ == '__main__':
    unittest.main()
